"""
Importing Plone data in JSON format to Wordpress via XML-RPC
M. Zimmermann, 2014/07, webmaster@incunabulum.de

Supported Types: ATFolder, ATDocument, ATNewsitem, ATBlob
Unsupported: ATTopic, ATLink

Wordpress XML-RPC Documentation: http://python-wordpress-xmlrpc.readthedocs.org/en/latest/overview.html
"""

import datetime
import json
import mimetypes
import os
import time
import xmlrpclib
from wordpress_xmlrpc import xmlrpc_client
from wordpress_xmlrpc.base import Client
from wordpress_xmlrpc.methods import media, posts
from wordpress_xmlrpc.wordpress import WordPressPage, WordPressPost


# #############################################################################
### Configuration
##############################################################################
ploneDataDirectory = '/root/p2wp/data'
zopePrefix = '/incunabulum_de_www'

wpSiteUrl = 'xxx/xmlrpc.php'
wpUser = 'xxx'
wpPwd = 'xxx'

createImagePages = False

mappingFile = '/root/p2wp/plone2WordpressMapping.csv'

##############################################################################
### Internal variables
##############################################################################
jsonFiles = []
dataFiles = []

contentTypes = {}
ploneUrl2JsonFile = {}
ploneUrl2WordpressID = {}
ploneUrl2WordpressUrl = {}
ploneUrl2WordpressBlobID = {}
ploneUrl2WordpressBlobUrl = {}
indexPages = []


##############################################################################
### Code
##############################################################################

def readPloneDataDirectory():
    for f in os.listdir(ploneDataDirectory):
        if '-file-' not in f:
            jsonFiles.append(f)
        else:
            dataFiles.append(f)


def findContentTypes():
    for f in jsonFiles:
        f = os.path.join(ploneDataDirectory, f)

        with open(f) as jsonFile:

            # parse json file
            jsonData = json.load(jsonFile)
            assert isinstance(jsonData, dict)

            # determine type
            if "_type" in jsonData.keys():
                cType = jsonData['_type']

                # create list in contentTypes if not already present
                if cType not in contentTypes.keys():
                    contentTypes[cType] = []

                # add file to contentType list
                contentTypes[cType].append(f)

            else:
                print("ERROR: No type attribute in json file: %s" % f)

            # store path to json file relation
            path = jsonData['_path']
            path = path.replace(zopePrefix, "")
            ploneUrl2JsonFile[path] = f

    print("Found content types: %s" % ", ".join(contentTypes.keys()))


def printContentTypeStatistics():
    print("Content types:")
    assert isinstance(contentTypes, dict)
    for key in contentTypes.keys():
        noOfItem = len(contentTypes[key])
        print("   %s: %d items" % (key, noOfItem))


def createWordpressPageOrPost(jsonData, pType):
    wpClient = Client(wpSiteUrl, wpUser, wpPwd)

    if pType == "Page":
        page = WordPressPage()
    else:
        page = WordPressPost()

    page.title = jsonData['title']

    # use plone ID as page slug
    page.slug = jsonData['id']

    # Pages do not support excerpts. We therefore add this to the top of the content.
    # page.excerpt = jsonData['description']
    if len(jsonData['description']) > 0:
        page.content = "<p><b>%s</b></p> %s" % (jsonData['description'], jsonData['text'])
    else:
        page.content = "%s" %  jsonData['text']

    page.date = createDateTime(jsonData['creation_date'])
    page.date_modified = createDateTime(jsonData['modification_date'])

    page.comment_status = 'closed'
    page.post_status = 'publish'

    # add categories to posts
    if pType == "Post":
        categories = jsonData['subject']
        page.terms_names = {
            # 'post_tag': categories,
            'category': categories,
        }


    # define parent "folder"
    parentPath = getParentURL(jsonData['_path'])
    if parentPath:
        parentId = ploneUrl2WordpressID[parentPath]
        page.parent_id = parentId

    # create page
    pageID = wpClient.call(posts.NewPost(page))

    if pType == "Page":
        print ("   Created page %s (id %s)" % (jsonData['_path'], pageID))
    else:
        print ("   Created post %s (id %s)" % (jsonData['_path'], pageID))

    # and retrieve create page
    page = wpClient.call(posts.GetPost(pageID))


    # store path and id of post
    ploneUrl2WordpressID[jsonData['_path']] = pageID
    ploneUrl2WordpressUrl[jsonData['_path']] = page.link


def createWordpressBlob(rpcData, jsonData):
    wpClient = Client(wpSiteUrl, wpUser, wpPwd)

    try:
        response = wpClient.call(media.UploadFile(rpcData))
    except Exception, e:
        print "ERROR handling media file %s (Exception: %s)" % (jsonData['_path'], str(e))
        return None

    print ("   Uploaded media file %s (id %s) with url %s (type: %s)" % (
        response['file'], response['id'], response['url'], response['type']))

    ploneUrl2WordpressBlobUrl[jsonData['_path']] = response['url']
    ploneUrl2WordpressBlobID[jsonData['_path']] = response['id']

    return response


def createDateTime(dtStrg):
    dtStrg = dtStrg[0:16]

    # from: http://flask.pocoo.org/snippets/96/
    dtObj = datetime.datetime.strptime(dtStrg, "%Y/%m/%d %H:%M")

    return dtObj


def stripZopePrefix(path):
    """ strip leading zope prefix """
    path = path.replace(zopePrefix, "")
    return path


def getParentURL(url):
    idx = url.rfind('/')
    url = url[0:idx]

    if len(url) == 0:
        return None

    return url


def readJsonFile(jsonPath):
    with open(jsonPath) as jsonFile:
        # parse json file
        jsonData = json.load(jsonFile)
        assert isinstance(jsonData, dict)

        # strip zope prefix
        path = stripZopePrefix(jsonData['_path'])
        jsonData['_path'] = path

        if not jsonData.has_key('text'):
            jsonData['text'] = ''

        if not jsonData.has_key('description'):
            jsonData['description'] = ''
    return jsonData


def fixBlobSuffix(name, fType):
    # not an image > do nothing
    if "image" not in fType:
        return name

    if "jpeg" in fType.lower():

        # image with valid suffix > do noting
        if name.lower().endswith("jpg") or name.lower().endswith("jpeg"):
            return name

        # add suffix
        name += ".jpg"

    if "png" in fType.lower():

        # image with valid suffix > do noting
        if name.lower().endswith("png"):
            return name

        # add suffix
        name += ".png"

    return name


def handleATBlobs(blobs):
    assert isinstance(blobs, list)

    print "Working on ATBlobs"

    # loop over all blob json files
    for blob in blobs:
        # read json file
        jsonData = readJsonFile(blob)

        # upload media file
        fileName = blob.replace(".json", ".json-file-1")

        # prepare request
        rpcData = {
            'name': jsonData['id'],
            'type': jsonData['_content_type']

        }

        # fix image name (must contain image extension for images)
        rpcData['name'] = fixBlobSuffix(rpcData['name'], rpcData['type'])

        # read binary file
        with open(fileName, 'rb') as f:
            rpcData['bits'] = xmlrpc_client.Binary(f.read())

        # upload file
        response = createWordpressBlob(rpcData, jsonData)

	# handle empty response (error uploading file)
	if not response:
            continue

        # only create content pages for images if wanted
        if createImagePages == False and "image" in response['type']:
            print ("   No content page created for image %s (id %s)" % (response['file'], response['id']))
            continue

        # create content page
        content = ""
        if jsonData['description']:
            content += "<p> %s </p>" % jsonData['description']

        # empty title is replaced by file name
        if jsonData['title'] == '':
            jsonData['title'] = response['file']
        

        if "image" in response['type']:
            # images
            imgUrl = response['url']
            baseUrl = imgUrl[0: imgUrl.rfind('.')]
            suffix = imgUrl[imgUrl.rfind('.') + 1:]

            txt = "<a href=\"%s\">" % response['url']
            txt += "<img class=\"aligncenter size-medium wp-image-%s\" " \
                   "src=\"%s-300x199.%s\" " \
                   "alt=\"%s\" width=\"300\" height=\"199\" />" % (response['id'], baseUrl, suffix, response['file'])
            txt += "</a>"

            content += txt
        else:
            # other files
            content += "<p>Download file: <a href=\"%s\">%s</a></p>" % (response['url'], response['file'])

        # and create page
        jsonData['text'] = content
        createWordpressPageOrPost(jsonData, "Page")


def handleATDocuments(documents, pType="Page"):
    assert isinstance(documents, list)

    print "Working on ATDocuments"

    # loop over all documents
    for document in documents:

        # read json data
        jsonData = readJsonFile(document)

        # do we have an index page? These are already handled in ATFolders
        if jsonData['_path'] in indexPages:
            print("   Index file skipped: %s" % jsonData['_path'])
            continue

        # create wordpress page
        createWordpressPageOrPost(jsonData, pType)


def handleATFolders(atFolders):
    assert isinstance(atFolders, list)

    print "Working on ATFolders"

    # all folders are stored in dict: folders[path] -> json
    folders = {}

    # read folder data from json files
    for atFolder in atFolders:
        jsonData = readJsonFile(atFolder)

        # add folder data. We use URL as key
        path = jsonData['_path']
        folders[path] = jsonData

    # handle folders
    for folderPath in sorted(folders):

        aFolder = folders[folderPath]

        # does folder have a default page?
        defaultPage = aFolder['_defaultpage']

        # special handling of folders with defined default page (and not a listing)
        if defaultPage is not None:
            # construct index file path
            indexPagePath = folderPath + "/" + defaultPage
            indexPages.append(indexPagePath)

            # parse index file
            indexJsonFile = ploneUrl2JsonFile[indexPagePath]
            with open(indexJsonFile) as jsonFile:
                indexPage = json.load(jsonFile)

            # replace folder data with data from default page
            aFolder['title'] = indexPage['title']
            aFolder['description'] = indexPage['description']
            aFolder['text'] = indexPage['text']
            aFolder['creators'] = indexPage['creators']
            aFolder['creation_date'] = indexPage['creation_date']
            aFolder['modification_date'] = indexPage['modification_date']
            aFolder['effectiveDate'] = indexPage['effectiveDate']

            # aFolder["_type"] = indexPage["_type"]
            # aFolder["_path"] = indexPage["_path"]


        # mark all folder without a default page. These show a directory listing in plone.
        if defaultPage is None:
            aFolder['text'] = "ERROR - Directory listing missing. (MZI)"

        createWordpressPageOrPost(jsonData=aFolder, pType="Page")


def writeMappings():
    with open(mappingFile,'w') as mapFile:

        # pages & posts - urls
        mapFile.write("\n\n")
        mapFile.write("# Pages & Posts - Plone URL <> Worpress URL\n")
        mapFile.write("########################################################\n")
        for key, value in ploneUrl2WordpressUrl.iteritems():
            mapFile.write("%s, %s\n" % (key, value))

        # pages & posts - ids
        mapFile.write("\n\n")
        mapFile.write("# Pages & Posts - Plone URL <> Worpress ID\n")
        mapFile.write("########################################################\n")
        for key, value in ploneUrl2WordpressID.iteritems():
            mapFile.write("%s, %s\n" % (key, value))

        # blobs - urls
        mapFile.write("\n\n")
        mapFile.write("# Blobs - Plone URL <> Worpress Blob URL\n")
        mapFile.write("########################################################\n")
        for key, value in ploneUrl2WordpressBlobUrl.iteritems():
            mapFile.write("%s, %s\n" % (key, value))

        # blobs - ids
        mapFile.write("\n\n")
        mapFile.write("# Blobs - Plone URL <> Worpress Blob ID\n")
        mapFile.write("########################################################\n")
        for key, value in ploneUrl2WordpressBlobID.iteritems():
            mapFile.write("%s, %s\n" % (key, value))


if __name__ == "__main__":
    # find json files
    readPloneDataDirectory()

    # find content types
    findContentTypes()
    printContentTypeStatistics()

    # handle folders
    handleATFolders(atFolders=contentTypes["ATFolder"])

    # handle blobs
    handleATBlobs(blobs=contentTypes["ATBlob"])

    # handle documents
    handleATDocuments(documents=contentTypes["ATDocument"])

    # handle news items (blog posts)
    handleATDocuments(documents=contentTypes["ATNewsItem"], pType="Post")


    # write generated mappings
    writeMappings()






